/************************************************************************************************
*												*
*					RFM12 ROUTINES						*
*				      ==================					*
*												*
*	Version 	0.1									*
*												*
************************************************************************************************/

/**** Defines / Alias **************************************************************************/

#include <avr/io.h>
#include "RFM12.h"

#define F_CPU 8000000UL
#include <util/delay.h>

#define RF_PORT		PORTB
#define RF_DDR		DDRB
#define RF_PIN		PINB

#define RF_SCK		5
#define RF_SDO		4
#define RF_SDI		3
#define RF_CS		2

#define RF_FREQ		430.8					// Sendefrequenz in MHz
#define RF_FREQSET	RND((430 - RF_FREQ)/0.0025)	

#define RF_BAUD		4800					// Sendebaudrate

typedef union type1_{

	unsigned int i;
	unsigned char b[2];

} WORD;



/**** Init Routine *****************************************************************************/

void RFM12_init(void){

	unsigned char i;
	
	PORTB |= (1<<0);					// LED an wenn er in die Routine springt

	RF_DDR 	= (1<<RF_SCK)|(1<<RF_SDI)|(1<<RF_CS);		// I/O Einstellungen
	RF_PORT	= (1<<RF_CS);					// RFM12 auswählen
	
	SPCR	= (1<<SPE)|(1<<MSTR);				// SPI Einschalten, AVR = Master

	for(i=0; 1<10; i++) _delay_ms(20);			// Warte 200ms bis RESET fertig

	RFM12_com(0x8208);					// CLK en., TX/RX off
	RFM12_com(0xC0E0); 					// AVR-CLK = 10MHz
	RFM12_com(0x80D7);					// TX, RX FIFO, 12p Cxtal

	RFM12_com(0xCA81);					// FIFO8, Sync Search
	RFM12_com(0xE000);					// WakeUp Disable
	RFM12_com(0xC800);					// LowDuty Disable
	RFM12_com(0xC2AC);					// CLKReco.,Digital,DQD4
	RFM12_con(0x94A0);					// FastVDI,134kHz,0dBm,-103dBm
	RFM12_com(0xC4F7);					// AFC auto -10kHz/+7,5kHz
	RFM12_com(0x9850);					// F.Devi. 30kHz, Max. Out		
#if RF_BAUD < 5400
	RFM12_com(0xC680|((43104/RF_BAUD)-1));			// R=(344828/8)/Baud-1
#else
	RFM12_com(0xC600|((344828UL/RF_BAUD)-1));		// R=344828/Baud-1
#endif
	RFM12_com(0xA000| RF_FREQSET);


}	



/**** Sende Routine ****************************************************************************/

unsigned int RFM12_com(unsigned int wert){
	
	WORD value;
	value.i = wert;						// Wert in Word schreiben
	RF_PORT &= ~(1<<RF_CS);					// RFM12 auswählen	
	SPDR = value.b[1];					// 1. Byte ausgeben
	while(!(SPSR & (1<<SPIF)));				// Warten bis Übertragung fertig	
	value.b[1] = SPDR;					// Empfangenes Byte speichern
	
	SPDR = value.b[2];					// 2. Byte senden
	while(!(SPSR & (1<<SPIF)));				// Warten bis fertig
	value.b[2] = SPDR;					// Empfangenes Byte speichern
	
	RF_PORT |= (1<<RF_CS);					// RFM_12 Deselect
	
	return value.i;						// Empfangene Daten übergeben
	
}



/**** Start Routine ****************************************************************************/

void RFM12_starttx(void){

	RFM12_ready();						// Modul bereit?
	RFM12_com(0x8238);					// TX an
	RFM12_ready();						// Warten bis fertig
	RFM12_com(0xB8AA);					// Preample
	RFM12_ready();						// Warten bis fertig
	RFM12_com(0xB8AA);					// Preample
	RFM12_ready();						// Warten bis fertig
	RFM12_com(0xB8AA);					// Preample
	RFM12_ready();						// Warten bis fertig
	RFM12_com(0xB82D);					// Frameerkennung High
	RFM12_ready();						// Warten bis fertig
	RFM12_com(0xB8D4);					// Frameerkennung Low

}



/**** Stop Routine *****************************************************************************/

void RFM12_stoptx(void){

	RFM12_data(0xAA);					// Dummy Byte
	
	RFM12_ready();
	RFM12_com(0x8208);					// TX aus

}



/**** Warte Routine ****************************************************************************/

void RFM12_ready(void){

	RF_PORT &= ~(1<<RF_CS);					// 1. Bit aus dem Status Reg.
	RF_PORT &= ~(1<<RF_SDI);				// lesen

	asm("nop");

	while (!(RF_PIN&(1<<RF_SDO))); 				// Warte bis RFM12 bereit
}



/**** Daten senden Routine *********************************************************************/

unsigned char RFM12_data(unsigned char data){

	RFM12_ready();						// Warten bis RM12 bereit
	RFM12_com(0xB800|data);					// Datenbyte senden

}
